require '4e_analysis'

describe DwarfFighter do
	it_should_behave_like "all creatures"
	before do; @creature = DwarfFighter.new	end
end

describe ReapingStrike do
	it_should_behave_like "all attacks"
	before do; @action = ReapingStrike.new(nil) end
end

describe SteelSerpentStrike do
	it_should_behave_like "all attacks"
	before do; @action = SteelSerpentStrike.new(nil) end
end

describe BruteStrike do
	it_should_behave_like "all attacks"
	before do; @action = BruteStrike.new(nil) end

	it "should be expended upon crit" do
		Dice.max do
			@action.act(TestCreature.new)
			@action.should be_expended
		end
	end
end

describe DwarvenResilience do
	it_should_behave_like "all actions"
	
	before do
		@actor = TestCreature.new(:max_hp => 100, :max_surges => 10)
		@action = DwarvenResilience.new(@actor)
	end
	
	it "should spend a healing surge" do
		@actor.damage(50)
		@action.act(nil)
		@actor.hp.should == 75
		@actor.surges.should == 9
	end
		
	it "should only be used when healing won't be wasted" do
		@action.priority.should == :do_not_use
		@actor.damage(24)
		@action.priority.should == :do_not_use
		@actor.damage(1)
		@action.priority.should == :normal
		@actor.damage(10)
		@action.priority.should == :normal
	end
	
	it "should have urgent priority when bloodied" do
		@actor.damage(70)
		@action.priority.should == :urgent
	end
	
	it "should only be usable once per encounter" do
		@actor.damage(90)
		@action.should_not be_expended
		@action.act(@actor)
		@action.should be_expended
	end

	it "should reset after a short rest" do
		@actor.damage(90)
		@action.act(@actor)
		@action.should be_expended
		@action.short_rest
		@action.should_not be_expended
	end
end