require '4e_analysis'

describe Report do
	before do
		@output = StringIO.new
		@report = Report.new(@output)
	end

	it "should support 'print'" do
		@report.print "something"
		@output.string.should == "something"
	end

	it "should support 'puts'" do
		@report.puts "something"
		@output.string.should == "something\n"
	end
	
	it "'print' should indent" do
		@report.puts "A" do
			@report.print "B"
		end
		@output.string.should == "A\n  B"
	end
	
	it "'print' should only indent once" do
		@report.puts do
			@report.print "a"
			@report.print "b"
		end
		@output.string.should == "\n  ab"
	end
	
	it "'puts' should only indent once" do
		@report.puts do
			@report.print "a"
			@report.puts "b"
		end
		@output.string.should == "\n  ab\n"
	end
	
	it "'puts' should print a blank line when no parm" do
		@report.puts
		@output.string.should == "\n"
	end
	
	it "should print headlines" do
		@report.headline "A"
		@output.string.should == "[b]A[/b]\n"
	end
	
	it "should indent in blocks" do
		@report.puts "A" do
			@report.puts "B"
		end
		@report.puts "C"
		@output.string.should == "A\n  B\nC\n"
	end
	
	it "should hide less-important information" do
		@report.hide "A" do
			@report.puts "B"
		end
		@report.puts "C"
		@output.string.should == "[sblock=A]\nB\n[/sblock]\nC\n"
	end
end
