require '4e_analysis'

class Action
	def initialize(actor=nil, report=Report.new)
		@actor = actor
		@report = report
	end

	def priority
		:normal
	end

	def act(opponent)
	end
	
	def short_rest
	end

	def keywords
	end
	
	def report
		report_name do
			report_frequency
			report_type
			report_details
		end
	end

private

	def report_name(&proc)
		@report.print name
		@report.print " (expended)" if expended?
		@report.puts ":", &proc
	end
	
	def report_frequency
	 	case frequency
	 		when :at_will then @report.print "At-Will"
	 		when :encounter then @report.print "Encounter"
	 		when :daily then @report.print "Daily"
	 		else raise NotImplementedError
	 	end
	 	@report.print " * #{keywords.join(", ")}" if keywords
	 	@report.puts
	end
	
	def report_type
		case action_type
			when :standard then @report.puts "Standard Action"
			when :move then @report.puts "Move Action"
			when :minor then @report.puts "Minor Action"
			else raise NotImplementedError
		end
	end	
end

module AtWillFrequency
	def frequency
		:at_will
	end
	
	def expended?
		false
	end
end

module EncounterFrequency
	def frequency
		:encounter
	end
	
	def expended?
		@expended
	end
	
	def act(opponent)
		@expended = true
		super
		@report.puts "#{name} has been expended." if @expended
	end
	
	def short_rest
		@report.puts "#{name} recharges." if @expended
		@expended = false
		super
	end
end

module DailyFrequency
	def frequency
		:daily
	end
	
	def expended?
		@expended
	end
	
	def act(opponent)
		@expended = true
		super
		@report.puts "#{name} has been expended." if @expended
	end
end

module Reliable
	def miss(opponent)
		@expended = false
		super
		@report.puts "#{name} was not expended (Reliable)."
	end
	
	def keywords
		["Reliable"]
	end
end

class Attack < Action
	def usable?
		true
	end

	def short_rest
	end

	def act(opponent)
		attack = attack_roll
		assert attack.rolls.count == 1, "only handles one attack roll"
		@report.puts "Attacks #{opponent} with #{attack} vs. #{opponent.ac} AC (#{attack.details})."
		
		hit = opponent.hit?(attack)
		if attack.rolls == [20] && hit then
			crit(opponent)
		elsif attack.rolls == [20] || (hit && attack.rolls != [1]) then
			hit(opponent)
		else
			miss(opponent)
		end
	end
	
	def crit(opponent)
		damage = crit_effect
		@report.puts "Crits for #{damage} damage (#{damage.details})."
		opponent.damage(damage)
	end
	
	def hit(opponent)
		damage = hit_effect
		@report.puts "Hits for #{damage} damage (#{damage.details})."
		opponent.damage(damage)
	end
	
	def miss(opponent)
		damage = miss_effect
		if (damage)
			@report.puts "Misses but does #{damage} damage (#{damage.details})."
			opponent.damage(damage)
		else
			@report.puts "Misses."
		end
	end
	
	def crit_effect
		result = 0
		Dice.max do
			result = hit_effect
		end
		return result
	end
	
	def miss_effect
	end
		
	def report_details
		@report.puts "Attack: #{attack_roll.dice} vs. AC"
		@report.puts "Hit: #{hit_effect.dice} damage"
		@report.puts "Miss: #{miss_effect.dice} damage" if miss_effect
	end
end

class TestAction < Action
	include AtWillFrequency
	
	def initialize(opts={})
		super(nil)
		@priority = opts[:priority] || :standard
		@expended = opts[:expended] || false
		@type = opts[:type] || :standard
		@usable = true if @usable == nil
	end

	def name; "!!! Test Action" end
	def action_type; @type end
	def has_acted?; @acted end
	def expended?; @expended end
	def priority; @priority end

	def act(opponent)
		@acted = true
	end
end

