require '4e_analysis'

PRECISION = 1

class Fight
	attr_reader :rounds

	def initialize(pc, monster)
		@rounds = 0
		until monster.dead? || pc.dead?
		    @rounds += 1
			pc.take_turn(monster)
			monster.take_turn(pc)
	   	end
	   	pc.short_rest unless pc.dead?
	end
end


def analyze(monster_level, runs)
	rounds = Histogram.new
	surges = Histogram.new
	survival = Histogram.new
	deaths = Histogram.new

	report = Report.new(STDOUT)
	
	pc = DwarfFighter.new(report)
	monster = Soldier.new(report)

	report.headline "#{pc} vs. #{monster}"
	report.puts
	report.hide "Combatants" do
		report.puts "[code]"
		pc.report
		report.puts
		monster.report
		report.puts "[/code]"
	end

	report.puts "[code]"
 	report.puts "Days simulated: #{runs}"

	exemplars = []
	runs.times do
		output = StringIO.new
		day_report = Report.new(output)
	
	    pc = DwarfFighter.new
	    fights = 0
	    5.times do
	    	day_report.puts "FIGHT ##{fights +1}:" do
			    monster = Soldier.new
				fight = Encounter.new(pc, monster, day_report)
				fight.simulate
				
				unless pc.dead?
					rounds.increment(fight.rounds)
					pc.short_rest
			    	fights += 1				
					day_report.puts
				end
			end
			break if pc.dead?
		end
		survival.increment(fights)
		surges.increment(pc.surges) unless pc.dead?
		if pc.dead? then deaths.increment(1) else deaths.increment(0) end
		
		exemplars[fights] = output.string
	end
	
	
	num_format = "%.#{PRECISION}f"
	
	puts
	puts "Avg fights survived: #{num_format % survival.average}"
	survival.print

	puts
	puts "Avg surges remaining (survivors only): #{num_format % surges.average} of #{pc.max_surges}"
	surges.print

	puts
	puts "Avg rounds per fight (survivors only): #{num_format % rounds.average}"
	rounds.print

	puts
	puts "Survival rate: #{num_format % (100-(deaths.average * 100))}%"
	puts "[/code]"
			
	exemplars.each_with_index do |exemplar, index|
		report.hide "Example of #{index} fight(s) survived:" do
			report.puts "[code]"
			report.puts exemplar
			report.puts "[/code]"
		end
	end
end
