require '4e_analysis'

class Dice
	include Comparable

	def self.factory(num, sides, opts = {})
		@@dice_logic.new(num, sides, opts)
	end
	
	def initialize(number, sides, opts = {})
		@brutal = opts[:brutal] || 0
		@number = number
		@sides = sides
		@opts = opts
		@bonus = 0
	end
	
	def roll
		unless @rolls
			@total_roll = 0
			@rolls = []
			@number.times do
				begin
					value = roll_one
					@rolls.push value
					raise Dice::ValueTooSmallError.new(value) if value < 1
					raise Dice::ValueTooLargeError.new(value) if value > @sides
				end until value > @brutal
				@total_roll += value
			end
		end
		return @total_roll + @bonus
	end
		
	def dice
		result = "#{@number}d#{@sides}"
		result += "[brutal #{@brutal}]" if @brutal > 0
		result += "+#{@bonus}" if @bonus > 0
		result += "#{@bonus}" if @bonus < 0
		return result
	end
	
	def bonus=(value)
		@bonus = value
	end
	
	def details
		"#{dice}; rolled #{rolls.join(", ")}"
	end
	
	def to_s
		"#{roll}"
	end
	
	def coerce(other)
		return other, roll
	end
		
	def +(value)
		roll
		result = clone
		result.bonus = @bonus + value
		return result
	end
	
	def -(value)
		return self + -value
	end
	
	def <=>(value)
		roll <=> value
	end
		
	def rolls
		roll unless @rolls
		@rolls
	end
end		
	
class MinimumDice < Dice
	def roll_one
		return @brutal + 1
	end
end

class MaximumDice < Dice
	def roll_one
		return @sides
	end
		
	def details
		"#{dice}; maximized"
	end
end

class FixedDice < Dice
	def roll_one
		@@roll_list.shift or raise Dice::NoMoreDiceError
	end
end	

class RandomDice < Dice
	def roll_one
		return 1 + rand(@sides)
	end
end

class Dice
	@@dice_logic = RandomDice

	def self.dice_logic
		@@dice_logic
	end
	
	def self.roll_list=(list)
		@@roll_list = list
	end
	

	def self.min(&proc)
		raise "Expected block in Dice.min" unless proc
		@@dice_logic = MinimumDice
		proc.call
		@@dice_logic = RandomDice
	end
	
	def self.max(&proc)
		raise "Expected block in Dice.max" unless proc
		@@dice_logic = MaximumDice
		proc.call
		@@dice_logic = RandomDice
	end
	
	def self.fix(list, &proc)
		raise "Expected block in Dice.fix" unless proc
		@@dice_logic = FixedDice
		@@roll_list = list
		proc.call
		@@dice_logic = RandomDice
	end
			
	class NoMoreDiceError < StandardError
	end
		
	class ValueTooSmallError < StandardError
		def initialize(value)
			super "Dice roll too small: #{value}"
		end
	end
	
	class ValueTooLargeError < StandardError
		def initialize(value)
			super "Dice roll too large: #{value}"
		end
	end
end

class Fixnum
	def dice
		"#{self}"
	end
	
	def rolls
		[self]
	end
	
	def details
		"static"
	end

	def d4(opts = {})
		Dice.factory(self, 4, opts)
	end
	
	def d6(opts = {})
		Dice.factory(self, 6, opts)
	end
	
	def d8(opts = {})
		Dice.factory(self, 8, opts)
	end
	
	def d10(opts = {})
		Dice.factory(self, 10, opts)
	end
	
	def d12(opts = {})
		Dice.factory(self, 12, opts)
	end
	
	def d20(opts = {})
		Dice.factory(self, 20, opts)
	end
	
	def d100(opts = {})
		Dice.factory(self, 100, opts)
	end
end
