require '4e_analysis'

class DwarfFighter < Creature
	def name; "Dwarf Fighter" end
	def max_hp; 27 end
	def max_surges; 10 end
	def ac; 17 end	
	def power_list; [BruteStrike, SteelSerpentStrike, ReapingStrike, DwarvenResilience] end
end

class DwarvenResilienceParent < Action
	def name; "Dwarven Resilience" end
	def action_type; :minor end
	
	def report_details
		@report.puts "Spend a healing surge and regain #{@actor.surge_value} hit points."
	end
	
	def priority
		hp_wasted = @actor.hp + @actor.surge_value > @actor.max_hp
		return :do_not_use if @actor.surges <= 0 || hp_wasted
		return :urgent if @actor.bloodied?
		return :normal
	end
	
	def usable?
		return false if @actor.surges <= 0
		return @actor.hp <= @actor.max_hp - @actor.surge_value
	end
	
	def act(opponent)
		@actor.spend_surge
	end
end

class DwarvenResilience < DwarvenResilienceParent
	include EncounterFrequency
end

class ReapingStrike < Attack
	include AtWillFrequency

	def name; "Reaping Strike" end
	def action_type; :standard end
	def attack_roll; 1.d20 + 7 end
	def hit_effect; 2.d6(:brutal => 1)+6 end
	def miss_effect; 4 end
end

class SureStrike < Attack
	include AtWillFrequency
	
	def name; "Sure Strike" end
	def action_type; :standard end
	def attack_roll; 1.d20 + 9 end
	def hit_effect; 2.d6(:brutal => 1)+2 end
end

class SteelSerpentStrike < Attack
	include EncounterFrequency

	def name; "Steel Serpent Strike" end
	def action_type; :standard end
	def attack_roll; 1.d20 + 7 end
	def hit_effect; 4.d6(:brutal => 1)+6 end
end

class BruteStrike < Attack
	include DailyFrequency
	include Reliable
	
	def name; "Brute Strike" end
	def action_type; :standard end
	def attack_roll; 1.d20+7 end
	def hit_effect; 6.d6(:brutal => 1)+6 end
end